@@ -41,6 +41,7 @@
 
 /* Internal includes, do not use in your programs */
 #include <OpenIPMI/internal/ipmi_malloc.h>
+#include "cmdlang.h"
 
 static void
 control_list_handler(ipmi_entity_t *entity, ipmi_control_t *control,
@@ -648,7 +649,10 @@ control_event_handler(ipmi_control_t *control,
     if (!evi) {
 	rv = ENOMEM;
 	errstr = "Out of memory";
-	goto out_err;
+	ipmi_cmdlang_global_err(control_name,
+				"cmd_control.c(ipmi_cmdlang_control_change)",
+				errstr, rv);
+	return IPMI_EVENT_NOT_HANDLED;
     }
 
     ipmi_cmdlang_out(evi, "Object Type", "Control");
@@ -672,14 +676,6 @@ control_event_handler(ipmi_control_t *control,
     }
     ipmi_cmdlang_cmd_info_put(evi);
     return IPMI_EVENT_NOT_HANDLED;
-
- out_err:
-    ipmi_cmdlang_global_err(control_name,
-			    "cmd_control.c(ipmi_cmdlang_control_change)",
-			    errstr, rv);
-    if (evi)
-	ipmi_cmdlang_cmd_info_put(evi);
-    return IPMI_EVENT_NOT_HANDLED;
 }
 
 void
@@ -699,7 +695,10 @@ ipmi_cmdlang_control_change(enum ipmi_update_e op,
     if (!evi) {
 	rv = ENOMEM;
 	errstr = "Out of memory";
-	goto out_err;
+	ipmi_cmdlang_global_err(control_name,
+				"cmd_control.c(ipmi_cmdlang_control_change)",
+				errstr, rv);
+	return;
     }
 
     ipmi_cmdlang_out(evi, "Object Type", "Control");
@@ -737,14 +736,6 @@ ipmi_cmdlang_control_change(enum ipmi_update_e op,
     }
 
     ipmi_cmdlang_cmd_info_put(evi);
-    return;
-
- out_err:
-    ipmi_cmdlang_global_err(control_name,
-			    "cmd_control.c(ipmi_cmdlang_control_change)",
-			    errstr, rv);
-    if (evi)
-	ipmi_cmdlang_cmd_info_put(evi);
 }
 
 static ipmi_cmdlang_cmd_t *control_cmds;
