@@ -43,6 +43,7 @@
 
 /* Internal includes, do not use in your programs */
 #include <OpenIPMI/internal/ipmi_malloc.h>
+#include "cmdlang.h"
 
 /* Don't pollute the namespace iwth ipmi_fru_t. */
 void ipmi_cmdlang_dump_fru_info(ipmi_cmd_info_t *cmd_info, ipmi_fru_t *fru);
@@ -229,8 +230,9 @@ fru_setval(ipmi_fru_t *fru, void *cb_data)
     enum ipmi_fru_data_type_e dtype;
     int             i;
     int             num;
-    char            *name;
+    const char      *name;
     int             ival;
+    double          dval;
     int             rv;
     int             len;
     unsigned char   *data;
@@ -309,6 +311,36 @@ fru_setval(ipmi_fru_t *fru, void *cb_data)
 	}
 	break;
 
+    case IPMI_FRU_DATA_BOOLEAN:
+	ipmi_cmdlang_get_bool(argv[curr_arg], &ival, cmd_info);
+	if (cmdlang->err) {
+	    cmdlang->errstr = "value invalid";
+	    goto out_err;
+	}
+	curr_arg++;
+	rv = ipmi_fru_set_int_val(fru, i, num, ival);
+	if (rv) {
+	    cmdlang->errstr = "value invalid";
+	    cmdlang->err = EINVAL;
+	    goto out_err;
+	}
+	break;
+
+    case IPMI_FRU_DATA_FLOAT:
+	ipmi_cmdlang_get_double(argv[curr_arg], &dval, cmd_info);
+	if (cmdlang->err) {
+	    cmdlang->errstr = "value invalid";
+	    goto out_err;
+	}
+	curr_arg++;
+	rv = ipmi_fru_set_float_val(fru, i, num, dval);
+	if (rv) {
+	    cmdlang->errstr = "value invalid";
+	    cmdlang->err = EINVAL;
+	    goto out_err;
+	}
+	break;
+
     case IPMI_FRU_DATA_ASCII:
     case IPMI_FRU_DATA_BINARY:
     case IPMI_FRU_DATA_UNICODE:
@@ -365,7 +397,7 @@ fru_setval(ipmi_fru_t *fru, void *cb_data)
 		curr_arg++;
 		j++;
 	    }
-	    rv = ipmi_fru_set_data_val(fru, i, num, dtype, data, len);
+	    rv = ipmi_fru_set_data_val(fru, i, num, dtype, (char *) data, len);
 	    ipmi_mem_free(data);
 	}
 	if (rv) {
