@@ -47,6 +47,7 @@
 
 /* Internal includes, do not use in your programs */
 #include <OpenIPMI/internal/ipmi_malloc.h>
+#include "cmdlang.h"
 
 static void
 mc_list_handler(ipmi_domain_t *domain, ipmi_mc_t *mc, void *cb_data)
@@ -712,7 +713,8 @@ sdrs_fetched(ipmi_sdr_info_t *sdrs,
 	cmdlang->location = "cmd_mc.c(sdrs_fetched)";
     }
     ipmi_cmdlang_cmd_info_put(cmd_info);
-    ipmi_sdr_info_destroy(sdrs, NULL, NULL);
+    if (sdrs)
+	ipmi_sdr_info_destroy(sdrs, NULL, NULL);
     ipmi_mem_free(info);
 }
 
@@ -1836,8 +1838,8 @@ set_user1(ipmi_mc_t        *mc,
 	/* Optional value, afaict there is no way to get this value. */
 	rv |= ipmi_user_set_enable(user, info->enable_val);
 	
-    rv = ipmi_mc_set_user(mc, info->channel, info->user, user,
-			  set_user2, info);
+    rv |= ipmi_mc_set_user(mc, info->channel, info->user, user,
+			   set_user2, info);
     if (rv) {
 	cmdlang->err = EINVAL;
 	cmdlang->errstr = "Error sending set user access cmd";
@@ -1997,7 +1999,8 @@ mc_user_set(ipmi_mc_t *mc, void *cb_data)
 		cmdlang->errstr = "no name value";
 		goto out_err;
 	    }
-	    strncpy(info->name, argv[curr_arg], 16);
+	    strncpy(info->name, argv[curr_arg], 15);
+	    info->name[15] = '\0';
 	} else if (strcmp(argv[curr_arg], "password") == 0) {
 	    info->pw_set = 1;
 	    curr_arg++;
@@ -2015,7 +2018,8 @@ mc_user_set(ipmi_mc_t *mc, void *cb_data)
 		cmdlang->errstr = "no name value";
 		goto out_err;
 	    }
-	    strncpy(info->pw, argv[curr_arg], 20);
+	    strncpy(info->pw, argv[curr_arg], 19);
+	    info->pw[19] = '\0';
 	} else {
 	    cmdlang->err = EINVAL;
 	    cmdlang->errstr = "invalid setting";
@@ -2056,7 +2060,10 @@ mc_active(ipmi_mc_t *mc, int active, void *cb_data)
     if (!evi) {
 	rv = ENOMEM;
 	errstr = "Out of memory";
-	goto out_err;
+	ipmi_cmdlang_global_err(mc_name,
+				"cmd_mc.c(mc_active)",
+				errstr, rv);
+	return;
     }
 
     ipmi_cmdlang_out(evi, "Object Type", "MC");
@@ -2065,14 +2072,6 @@ mc_active(ipmi_mc_t *mc, int active, void *cb_data)
     ipmi_cmdlang_out_bool(evi, "Active", active);
 
     ipmi_cmdlang_cmd_info_put(evi);
-    return;
-
- out_err:
-    ipmi_cmdlang_global_err(mc_name,
-			    "cmd_mc.c(mc_active)",
-			    errstr, rv);
-    if (evi)
-	ipmi_cmdlang_cmd_info_put(evi);
 }
 
 static void
@@ -2089,7 +2088,10 @@ mc_fully_up(ipmi_mc_t *mc, void *cb_data)
     if (!evi) {
 	rv = ENOMEM;
 	errstr = "Out of memory";
-	goto out_err;
+	ipmi_cmdlang_global_err(mc_name,
+				"cmd_mc.c(mc_fully_up)",
+				errstr, rv);
+	return;
     }
 
     ipmi_cmdlang_out(evi, "Object Type", "MC");
@@ -2097,14 +2099,6 @@ mc_fully_up(ipmi_mc_t *mc, void *cb_data)
     ipmi_cmdlang_out(evi, "Operation", "Fully Up");
 
     ipmi_cmdlang_cmd_info_put(evi);
-    return;
-
- out_err:
-    ipmi_cmdlang_global_err(mc_name,
-			    "cmd_mc.c(mc_fully_up)",
-			    errstr, rv);
-    if (evi)
-	ipmi_cmdlang_cmd_info_put(evi);
 }
 
 void
