@@ -60,7 +60,7 @@ static GDBM_FILE gdbmf;
 #endif
 
 
-extern selector_t *debug_sel;
+extern struct selector_s *debug_sel;
 
 #ifdef IPMI_CHECK_LOCKS
 static void check_no_locks(os_handler_t *handler);
@@ -152,7 +152,7 @@ struct os_hnd_timer_id_s
 };
 
 static void
-timer_handler(selector_t  *sel,
+timer_handler(struct selector_s *sel,
 	      sel_timer_t *timer,
 	      void        *data)
 {
