@@ -34,6 +34,7 @@
 #ifndef OPENIPMI_GLIB_H
 #define OPENIPMI_GLIB_H
 
+#include <OpenIPMI/dllvisibility.h>
 #include <OpenIPMI/os_handler.h>
 
 #ifdef __cplusplus
@@ -44,11 +45,13 @@ extern "C" {
    set_log_handler in the os_handler. */
 
 /* Allocate and configure an OS handler. */
+IPMI_DLL_PUBLIC
 os_handler_t *ipmi_glib_get_os_handler(int priority);
 
 /* This is a convenience log handler that allows the glib stuff to be
    used without having to link with glib to get the log handler
    information. */
+IPMI_DLL_PUBLIC
 void ipmi_glib_set_log_handler(void (*hndlr)(const char *domain,
 					     const char *pfx,
 					     const char *msg));
