@@ -122,6 +122,7 @@ struct sensor_s
     unsigned int  scanning_enabled : 1;
     unsigned int  events_enabled   : 1;
     unsigned int  enabled          : 1;
+    unsigned int  data_ready	   : 1; /* Do we have valid data yet? */
 
     unsigned char sensor_type;
     unsigned char event_reading_code;
@@ -198,8 +199,27 @@ typedef struct led_data_s
     unsigned char def_override_color;
 } led_data_t;
 
+struct seq_entry {
+    int inuse;
+    void *src_addr;
+    unsigned int src_len;
+    uint8_t daddr;
+    uint8_t dlun;
+    uint8_t saddr;
+    uint8_t slun;
+    uint8_t chan_num;
+    uint8_t orig_seq;
+    uint8_t cmd;
+    uint32_t sid;
+
+    /* Number of ticks (seconds) before we give up on this message. */
+    unsigned int time_to_live;
+};
+
 struct lmc_data_s
 {
+    sys_data_t *sys;
+
     emu_data_t *emu;
 
     char enabled;
@@ -212,6 +232,9 @@ struct lmc_data_s
 
     channel_t *channels[IPMI_MAX_CHANNELS];
 
+    struct seq_entry seq_entries[64];
+    unsigned int next_seq;
+
     channel_t sys_channel;
     channel_t ipmb_channel;
 
@@ -225,6 +248,9 @@ struct lmc_data_s
     ipmi_child_quit_t child_quit_handler;
     startcmd_t startcmd;
 
+    struct msg_q recv_q;
+    unsigned int recv_q_len; /* How many items in the receive queue? */
+
     unsigned char evq[16];
     char  ev_in_q;
 
@@ -264,8 +290,6 @@ struct lmc_data_s
 					   (1 << IPMI_MC_EVENT_MSG_BUF_BIT))
     unsigned char global_enables;
 
-    sys_data_t *sysinfo;
-
     sel_t sel;
 
     sdrs_t main_sdrs;
@@ -332,48 +356,6 @@ struct lmc_data_s
     ipmi_timer_t *watchdog_timer;
 };
 
-typedef struct atca_site_s
-{
-    unsigned char valid;
-    unsigned char hw_address;
-    unsigned char site_type;
-    unsigned char site_number;
-} atca_site_t;
-
-#define MAX_EMU_ADDR		16
-#define MAX_EMU_ADDR_DATA	64
-typedef struct emu_addr_s
-{
-    unsigned char valid;
-    unsigned char addr_type;
-    unsigned char addr_data[MAX_EMU_ADDR_DATA];
-    unsigned int  addr_len;
-} emu_addr_t;
-
-struct emu_data_s
-{
-    sys_data_t *sysinfo;
-
-    int users_changed;
-
-    int          atca_mode;
-    atca_site_t  atca_sites[128]; /* Indexed by HW address. */
-    uint32_t     atca_fru_inv_curr_timestamp;
-    uint16_t     atca_fru_inv_curr_lock_id;
-    int          atca_fru_inv_locked;
-    int          atca_fru_inv_lock_timeout;
-
-    unsigned char *temp_fru_inv_data;
-    unsigned int  temp_fru_inv_data_len;
-
-    void *user_data;
-
-    ipmi_emu_sleep_cb sleeper;
-
-    struct timeval last_addr_change_time;
-    emu_addr_t addr[MAX_EMU_ADDR];
-};
-
 /* Device ID support bits */
 #define IPMI_DEVID_CHASSIS_DEVICE	(1 << 7)
 #define IPMI_DEVID_BRIDGE		(1 << 6)
@@ -392,7 +374,7 @@ sdr_t *find_sdr_by_recid(sdrs_t     *sdrs,
 			 uint16_t   record_id,
 			 sdr_t      **prev);
 
-sdr_t *new_sdr_entry(sdrs_t *sdrs, unsigned char length);
+sdr_t *new_sdr_entry(lmc_data_t *mc, sdrs_t *sdrs, unsigned char length);
 void add_sdr_entry(lmc_data_t *mc, sdrs_t *sdrs, sdr_t *entry);
 void read_mc_sdrs(lmc_data_t *mc, sdrs_t *sdrs, const char *sdrtype);
 
@@ -425,8 +407,16 @@ extern cmd_handler_f oem0_netfn_handlers[256];
 #define set_bit(m, b, v) (m) = (v) ? ((m) | (1 << (b))) : ((m) & ~(1 << (b)))
 #define bit_set(m, b) (!!((m) & (1 << (b))))
 
+void ipmi_mc_handle_msg(lmc_data_t *srcmc, msg_t *msg);
 
 int ipmi_mc_is_power_on(lmc_data_t *mc);
 
+int reserve_mc_seq(lmc_data_t *mc, msg_t *msg, 
+		   unsigned char *rdata,
+		   unsigned int  *rdata_len);
+
+int find_mc_seq(lmc_data_t *mc, msg_t *msg,
+		unsigned char *rdata,
+		unsigned int  *rdata_len);
 
 #endif /* __BMC_H_ */
