@@ -53,7 +53,6 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
-#include <malloc.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -62,6 +61,7 @@
 #include <OpenIPMI/ipmi_err.h>
 #include <OpenIPMI/ipmi_msgbits.h>
 #include <OpenIPMI/ipmi_bits.h>
+#include <OpenIPMI/serv.h>
 
 static void sensor_poll(void *cb_data);
 
@@ -105,6 +105,34 @@ handle_set_event_receiver(lmc_data_t    *mc,
     *rdata_len = 1;
 }
 
+static void
+handle_platform_event(lmc_data_t    *mc,
+		      msg_t         *msg,
+		      unsigned char *rdata,
+		      unsigned int  *rdata_len,
+		      void          *cb_data)
+{
+    unsigned char sel_data[13];
+
+    if (check_msg_length(msg, 8, rdata, rdata_len))
+	return;
+
+    sel_data[0] = 0;
+    sel_data[1] = 0;
+    sel_data[2] = 0;
+    sel_data[3] = 0;
+    sel_data[4] = msg->data[0];
+    sel_data[5] = msg->orig_channel->channel_num << 4;
+    sel_data[6] = msg->data[1];
+    sel_data[7] = msg->data[2];
+    sel_data[8] = msg->data[3];
+    sel_data[9] = msg->data[4];
+    sel_data[10] = msg->data[5];
+    sel_data[11] = msg->data[6];
+    sel_data[12] = msg->data[7];
+    mc_new_event(mc, 2, sel_data);
+}
+
 static void
 handle_get_device_sdr_info(lmc_data_t    *mc,
 			   msg_t         *msg,
@@ -118,7 +146,7 @@ handle_get_device_sdr_info(lmc_data_t    *mc,
     }
 
     rdata[0] = 0;
-    rdata[1] = mc->num_sensors_per_lun[msg->rs_lun];
+    rdata[1] = mc->num_sensors_per_lun[msg->dlun];
     rdata[2] = ((mc->dynamic_sensor_population << 7)
 		| (mc->lun_has_sensors[3] << 3)
 		| (mc->lun_has_sensors[2] << 2)
@@ -150,12 +178,12 @@ handle_reserve_device_sdr_repository(lmc_data_t    *mc,
 	return;
     }
 
-    mc->device_sdrs[msg->rs_lun].reservation++;
-    if (mc->device_sdrs[msg->rs_lun].reservation == 0)
-	mc->device_sdrs[msg->rs_lun].reservation++;
+    mc->device_sdrs[msg->dlun].reservation++;
+    if (mc->device_sdrs[msg->dlun].reservation == 0)
+	mc->device_sdrs[msg->dlun].reservation++;
 
     rdata[0] = 0;
-    ipmi_set_uint16(rdata+1, mc->device_sdrs[msg->rs_lun].reservation);
+    ipmi_set_uint16(rdata+1, mc->device_sdrs[msg->dlun].reservation);
     *rdata_len = 3;
 }
 
@@ -183,7 +211,7 @@ handle_get_device_sdr(lmc_data_t    *mc,
 	uint16_t reservation = ipmi_get_uint16(msg->data+0);
 
 	if ((reservation != 0)
-	    && (reservation != mc->device_sdrs[msg->rs_lun].reservation))
+	    && (reservation != mc->device_sdrs[msg->dlun].reservation))
 	{
 	    rdata[0] = IPMI_INVALID_RESERVATION_CC;
 	    *rdata_len = 1;
@@ -196,16 +224,16 @@ handle_get_device_sdr(lmc_data_t    *mc,
     count = msg->data[5];
 
     if (record_id == 0) {
-	entry = mc->device_sdrs[msg->rs_lun].sdrs;
+	entry = mc->device_sdrs[msg->dlun].sdrs;
     } else if (record_id == 0xffff) {
-	entry = mc->device_sdrs[msg->rs_lun].sdrs;
+	entry = mc->device_sdrs[msg->dlun].sdrs;
 	if (entry) {
 	    while (entry->next) {
 		entry = entry->next;
 	    }
 	}
     } else {
-	entry = find_sdr_by_recid(&mc->device_sdrs[msg->rs_lun],
+	entry = find_sdr_by_recid(&mc->device_sdrs[msg->dlun],
 				  record_id, NULL);
     }
 
@@ -256,13 +284,13 @@ handle_set_sensor_hysteresis(lmc_data_t    *mc,
 	return;
 
     sens_num = msg->data[0];
-    if ((sens_num >= 255) || (!mc->sensors[msg->rs_lun][sens_num])) {
+    if ((sens_num >= 255) || (!mc->sensors[msg->dlun][sens_num])) {
 	rdata[0] = IPMI_INVALID_DATA_FIELD_CC;
 	*rdata_len = 1;
 	return;
     }
 
-    sensor = mc->sensors[msg->rs_lun][sens_num];
+    sensor = mc->sensors[msg->dlun][sens_num];
     if (sensor->hysteresis_support != IPMI_HYSTERESIS_SUPPORT_SETTABLE) {
 	rdata[0] = IPMI_INVALID_CMD_CC;
 	*rdata_len = 1;
@@ -290,13 +318,13 @@ handle_get_sensor_hysteresis(lmc_data_t    *mc,
 	return;
 
     sens_num = msg->data[0];
-    if ((sens_num >= 255) || (!mc->sensors[msg->rs_lun][sens_num])) {
+    if ((sens_num >= 255) || (!mc->sensors[msg->dlun][sens_num])) {
 	rdata[0] = IPMI_INVALID_DATA_FIELD_CC;
 	*rdata_len = 1;
 	return;
     }
 
-    sensor = mc->sensors[msg->rs_lun][sens_num];
+    sensor = mc->sensors[msg->dlun][sens_num];
     if ((sensor->hysteresis_support != IPMI_HYSTERESIS_SUPPORT_SETTABLE)
 	&& (sensor->hysteresis_support != IPMI_HYSTERESIS_SUPPORT_READABLE))
     {
@@ -460,13 +488,13 @@ handle_set_sensor_thresholds(lmc_data_t    *mc,
 	return;
 
     sens_num = msg->data[0];
-    if ((sens_num >= 255) || (!mc->sensors[msg->rs_lun][sens_num])) {
+    if ((sens_num >= 255) || (!mc->sensors[msg->dlun][sens_num])) {
 	rdata[0] = IPMI_INVALID_DATA_FIELD_CC;
 	*rdata_len = 1;
 	return;
     }
 
-    sensor = mc->sensors[msg->rs_lun][sens_num];
+    sensor = mc->sensors[msg->dlun][sens_num];
     if ((sensor->event_reading_code != IPMI_EVENT_READING_TYPE_THRESHOLD)
 	|| (sensor->threshold_support != IPMI_THRESHOLD_ACCESS_SUPPORT_SETTABLE))
     {
@@ -511,13 +539,13 @@ handle_get_sensor_thresholds(lmc_data_t    *mc,
 	return;
 
     sens_num = msg->data[0];
-    if ((sens_num >= 255) || (!mc->sensors[msg->rs_lun][sens_num])) {
+    if ((sens_num >= 255) || (!mc->sensors[msg->dlun][sens_num])) {
 	rdata[0] = IPMI_INVALID_DATA_FIELD_CC;
 	*rdata_len = 1;
 	return;
     }
 
-    sensor = mc->sensors[msg->rs_lun][sens_num];
+    sensor = mc->sensors[msg->dlun][sens_num];
     if ((sensor->event_reading_code != IPMI_EVENT_READING_TYPE_THRESHOLD)
 	|| ((sensor->threshold_support != IPMI_THRESHOLD_ACCESS_SUPPORT_SETTABLE)
 	    && (sensor->threshold_support != IPMI_THRESHOLD_ACCESS_SUPPORT_READABLE)))
@@ -556,15 +584,15 @@ handle_set_sensor_event_enable(lmc_data_t    *mc,
 	return;
 
     sens_num = msg->data[0];
-    if ((sens_num >= 255) || (!mc->sensors[msg->rs_lun][sens_num])) {
+    if ((sens_num >= 255) || (!mc->sensors[msg->dlun][sens_num])) {
 	rdata[0] = IPMI_INVALID_DATA_FIELD_CC;
 	*rdata_len = 1;
 	return;
     }
 
-    sensor = mc->sensors[msg->rs_lun][sens_num];
+    sensor = mc->sensors[msg->dlun][sens_num];
     if ((sensor->event_support == IPMI_EVENT_SUPPORT_NONE)
-	|| (sensor->event_support == IPMI_EVENT_SUPPORT_GLOBAL_ENABLE))
+	|| (sensor->event_support == IPMI_EVENT_SUPPORT_GLOBAL_DISABLE))
     {
 	rdata[0] = IPMI_INVALID_CMD_CC;
 	*rdata_len = 1;
@@ -643,15 +671,15 @@ handle_get_sensor_event_enable(lmc_data_t    *mc,
 	return;
 
     sens_num = msg->data[0];
-    if ((sens_num >= 255) || (!mc->sensors[msg->rs_lun][sens_num])) {
+    if ((sens_num >= 255) || (!mc->sensors[msg->dlun][sens_num])) {
 	rdata[0] = IPMI_INVALID_DATA_FIELD_CC;
 	*rdata_len = 1;
 	return;
     }
 
-    sensor = mc->sensors[msg->rs_lun][sens_num];
+    sensor = mc->sensors[msg->dlun][sens_num];
     if ((sensor->event_support == IPMI_EVENT_SUPPORT_NONE)
-	|| (sensor->event_support == IPMI_EVENT_SUPPORT_GLOBAL_ENABLE))
+	|| (sensor->event_support == IPMI_EVENT_SUPPORT_GLOBAL_DISABLE))
     {
 	rdata[0] = IPMI_INVALID_CMD_CC;
 	*rdata_len = 1;
@@ -699,13 +727,13 @@ handle_get_sensor_type(lmc_data_t    *mc,
 	return;
 
     sens_num = msg->data[0];
-    if ((sens_num >= 255) || (!mc->sensors[msg->rs_lun][sens_num])) {
+    if ((sens_num >= 255) || (!mc->sensors[msg->dlun][sens_num])) {
 	rdata[0] = IPMI_INVALID_DATA_FIELD_CC;
 	*rdata_len = 1;
 	return;
     }
 
-    sensor = mc->sensors[msg->rs_lun][sens_num];
+    sensor = mc->sensors[msg->dlun][sens_num];
     rdata[0] = 0;
     rdata[1] = sensor->sensor_type;
     rdata[2] = sensor->event_reading_code;
@@ -726,13 +754,13 @@ handle_get_sensor_reading(lmc_data_t    *mc,
 	return;
 
     sens_num = msg->data[0];
-    if ((sens_num >= 255) || (!mc->sensors[msg->rs_lun][sens_num])) {
+    if ((sens_num >= 255) || (!mc->sensors[msg->dlun][sens_num])) {
 	rdata[0] = IPMI_INVALID_DATA_FIELD_CC;
 	*rdata_len = 1;
 	return;
     }
 
-    sensor = mc->sensors[msg->rs_lun][sens_num];
+    sensor = mc->sensors[msg->dlun][sens_num];
 
     if (sensor->event_only) {
 	rdata[0] = IPMI_INVALID_DATA_FIELD_CC;
@@ -740,6 +768,12 @@ handle_get_sensor_reading(lmc_data_t    *mc,
 	return;
     }
 
+    if (!sensor->data_ready) {
+	rdata[0] = IPMI_COULD_NOT_PROVIDE_RESPONSE_CC;
+	*rdata_len = 1;
+	return;
+    }
+
     rdata[0] = 0;
     rdata[1] = sensor->value;
     rdata[2] = ((sensor->events_enabled << 7)
@@ -766,13 +800,13 @@ handle_rearm_sensor_events(lmc_data_t    *mc,
 	return;
 
     sens_num = msg->data[0];
-    if ((sens_num >= 255) || (!mc->sensors[msg->rs_lun][sens_num])) {
+    if ((sens_num >= 255) || (!mc->sensors[msg->dlun][sens_num])) {
 	rdata[0] = IPMI_INVALID_DATA_FIELD_CC;
 	*rdata_len = 1;
 	return;
     }
 
-    sensor = mc->sensors[msg->rs_lun][sens_num];
+    sensor = mc->sensors[msg->dlun][sens_num];
 
     if (msg->data[1] & (1 << 7)) {
 	rearm_assert = 0x7fff;
@@ -1116,7 +1150,7 @@ static int check_sensor_sdr(lmc_data_t *mc, unsigned char *sdr,
 			    unsigned int len, void *cb_data)
 {
     sensor_t *sensor = cb_data;
-    uint8_t mc_ipmb = ipmi_mc_get_ipmb(mc);
+    uint8_t mc_ipmb = is_mc_get_ipmb(mc);
 
     if (len < 8)
 	return 0;
@@ -1148,10 +1182,9 @@ ipmi_mc_add_sensor(lmc_data_t    *mc,
     if ((lun >= 4) || (sens_num >= 255) || (mc->sensors[lun][sens_num]))
 	return EINVAL;
 
-    sensor = malloc(sizeof(*sensor));
+    sensor = mc->sys->alloc(mc->sys, sizeof(*sensor));
     if (!sensor)
 	return ENOMEM;
-    memset(sensor, 0, sizeof(*sensor));
 
     sensor->mc = mc;
     sensor->lun = lun;
@@ -1159,6 +1192,7 @@ ipmi_mc_add_sensor(lmc_data_t    *mc,
     sensor->sensor_type = type;
     sensor->event_reading_code = event_reading_code;
     sensor->event_only = event_only;
+    sensor->data_ready = 0;
     mc->sensors[lun][sens_num] = sensor;
 
     if (mc->emu->atca_mode && (type == 0xf0)) {
@@ -1322,15 +1356,14 @@ file_init(lmc_data_t *mc,
 	return EINVAL;
     }
 
-    err = get_delim_str(toks, &fname, errstr);
+    err = get_delim_str(mc->sys, toks, &fname, errstr);
     if (err)
 	return ENOMEM;
-    f = malloc(sizeof(*f));
+    f = mc->sys->alloc(mc->sys, sizeof(*f));
     if (!f) {
-	free(fname);
+	mc->sys->free(mc->sys, fname);
 	return ENOMEM;
     }
-    memset(f, 0, sizeof(*f));
     f->emu = mc->emu;
     f->sensor_mc = mc;
     f->sensor_lun = lun;
@@ -1448,8 +1481,8 @@ file_init(lmc_data_t *mc,
     return 0;
 
   out_err:
-    free(fname);
-    free(f);
+    mc->sys->free(mc->sys, fname);
+    mc->sys->free(mc->sys, f);
     return -1;
 }
 
@@ -1506,7 +1539,7 @@ static void
 free_sensor(lmc_data_t *mc, sensor_t *sensor)
 {
     mc->sensors[sensor->lun][sensor->num] = NULL;
-    free(sensor);
+    mc->sys->free(mc->sys, sensor);
 }
 
 static void
@@ -1522,10 +1555,10 @@ sensor_poll(void *cb_data)
 
 	err = sensor->poll(sensor->cb_data, &val, &errstr);
 	if (err) {
-	    mc->sysinfo->log(mc->sysinfo, OS_ERROR, NULL,
-			     "Error getting sensor value (%2.2x,%d,%d): %s, %s",
-			     ipmi_mc_get_ipmb(mc), sensor->lun, sensor->num,
-			     strerror(err), errstr);
+	    mc->sys->log(mc->sys, OS_ERROR, NULL,
+			 "Error getting sensor value (%2.2x,%d,%d): %s, %s",
+			 is_mc_get_ipmb(mc), sensor->lun, sensor->num,
+			 strerror(err), errstr);
 	    goto out_restart;
 	}
 	
@@ -1536,13 +1569,18 @@ sensor_poll(void *cb_data)
 	} else {
 	    unsigned int i;
 	    
+	    /*
+	     * Technically the sensor value on discrete sensors is ignored,
+	     * but throw the low 8 bits in there for good measure.
+	     */
+	    sensor->value = (unsigned char) val;
 	    for (i = 0; i < 15; i++)
 		set_sensor_bit(mc, sensor,
 			       i, ((val >> i) & 1), 0, 0xff, 0xff, 1);
 	}
-
+	sensor->data_ready = 1;
       out_restart:
-	mc->sysinfo->start_timer(sensor->poll_timer, &sensor->poll_timer_time);
+	mc->sys->start_timer(sensor->poll_timer, &sensor->poll_timer_time);
     }
 }
 
@@ -1570,15 +1608,15 @@ ipmi_mc_add_polled_sensor(lmc_data_t    *mc,
     sensor->poll_timer_time.tv_sec = poll_rate / 1000;
     sensor->poll_timer_time.tv_usec = (poll_rate % 1000) * 1000;
     sensor->cb_data = cb_data;
-    
-    err = mc->sysinfo->alloc_timer(mc->sysinfo, sensor_poll, sensor,
-				   &sensor->poll_timer);
+
+    err = mc->sys->alloc_timer(mc->sys, sensor_poll, sensor,
+			       &sensor->poll_timer);
     if (err) {
 	free_sensor(mc, sensor);
 	return err;
     }
 
-    mc->sysinfo->start_timer(sensor->poll_timer, &sensor->poll_timer_time);
+    mc->sys->start_timer(sensor->poll_timer, &sensor->poll_timer_time);
 
     return 0;
 }
@@ -1590,7 +1628,7 @@ handle_ipmi_get_pef_capabilities(lmc_data_t    *mc,
 				 unsigned int  *rdata_len,
 				 void          *cb_data)
 {
-    if (!mc->sysinfo) {
+    if (!mc->sys) {
 	rdata[0] = IPMI_INVALID_CMD_CC;
 	*rdata_len = 1;
 	return;
@@ -1612,7 +1650,7 @@ handle_ipmi_set_pef_config_parms(lmc_data_t    *mc,
 {
     unsigned char err = 0;
     int           set, block;
-    sys_data_t    *sys = mc->sysinfo;
+    sys_data_t    *sys = mc->sys;
 
     if (msg->len < 2) {
 	rdata[0] = IPMI_REQUEST_DATA_LENGTH_INVALID_CC;
@@ -1792,8 +1830,7 @@ handle_ipmi_get_pef_config_parms(lmc_data_t    *mc,
     unsigned int  length = 0;
     unsigned char err = 0;
     unsigned char tmpdata[18];
-
-    sys_data_t    *sys = mc->sysinfo;
+    sys_data_t    *sys = mc->sys;
 
     if (msg->len < 3) {
 	rdata[0] = IPMI_REQUEST_DATA_LENGTH_INVALID_CC;
@@ -1931,6 +1968,7 @@ handle_ipmi_get_pef_config_parms(lmc_data_t    *mc,
 cmd_handler_f sensor_event_netfn_handlers[256] = {
     [IPMI_GET_EVENT_RECEIVER_CMD] = handle_get_event_receiver,
     [IPMI_SET_EVENT_RECEIVER_CMD] = handle_set_event_receiver,
+    [IPMI_PLATFORM_EVENT_CMD] = handle_platform_event,
     [IPMI_GET_DEVICE_SDR_INFO_CMD] = handle_get_device_sdr_info,
     [IPMI_RESERVE_DEVICE_SDR_REPOSITORY_CMD] = handle_reserve_device_sdr_repository,
     [IPMI_GET_DEVICE_SDR_CMD] = handle_get_device_sdr,
