@@ -61,7 +61,6 @@
 #include <netdb.h>
 #include <dlfcn.h>
 
-#include <OpenIPMI/serv.h>
 #include <OpenIPMI/lanserv.h>
 #include <OpenIPMI/serserv.h>
 #include <OpenIPMI/ipmbserv.h>
@@ -81,11 +80,11 @@ read_persist_users(sys_data_t *sys)
 	if (!mc)
 	    continue;
 
-	p = read_persist("users.mc%2.2x", ipmi_mc_get_ipmb(mc));
+	p = read_persist(sys, "users.mc%2.2x", sys->mc_get_ipmb(mc));
 	if (!p)
 	    continue;
 
-	users = ipmi_mc_get_users(mc);
+	users = sys->mc_get_users(mc);
 	for (j = 0; j <= MAX_USERS; j++) {
 	    void *data;
 	    unsigned int len;
@@ -99,12 +98,12 @@ read_persist_users(sys_data_t *sys)
 	    if (!read_persist_data(p, &data, &len, "%d.username", j)) {
 		if (len == sizeof(users[j].username))
 		    memcpy(users[j].username, data, len);
-		free_persist_data(data);
+		free_persist_data(p, data);
 	    }
 	    if (!read_persist_data(p, &data, &len, "%d.passwd", j)) {
 		if (len == sizeof(users[j].pw))
 		    memcpy(users[j].pw, data, len);
-		free_persist_data(data);
+		free_persist_data(p, data);
 	    }
 	    if (!read_persist_int(p, &iv, "%d.privilege", j))
 		users[j].privilege = iv;
@@ -125,14 +124,14 @@ write_persist_users(sys_data_t *sys)
 	user_t *users;
 	persist_t *p;
 
-	if (!mc || !ipmi_mc_users_changed(mc))
+	if (!mc || !sys->mc_users_changed(mc))
 	    continue;
 
-	p = alloc_persist("users.mc%2.2x", ipmi_mc_get_ipmb(mc));
+	p = alloc_persist(sys, "users.mc%2.2x", sys->mc_get_ipmb(mc));
 	if (!p)
 	    return ENOMEM;
 
-	users = ipmi_mc_get_users(mc);
+	users = sys->mc_get_users(mc);
 	for (j = 0; j <= MAX_USERS; j++) {
 	    add_persist_int(p, users[j].valid, "%d.valid", j);
 	    add_persist_int(p, users[j].link_auth, "%d.link_auth", j);
@@ -160,7 +159,7 @@ struct variable {
  *
  */
 int
-add_variable(const char *name, char *value)
+add_variable(sys_data_t *sys, const char *name, char *value)
 {
     struct variable *var = vars, *last = NULL;
 
@@ -171,14 +170,14 @@ add_variable(const char *name, char *value)
 	var = var->next;
     }
     if (var) {
-	free(var->value);
+	sys->free(sys, var->value);
     } else {
-	var = malloc(sizeof(*var));
+	var = sys->alloc(sys, sizeof(*var));
 	if (!var)
 	    return ENOMEM;
-	var->name = strdup(name);
+	var->name = sys_strdup(sys, name);
 	if (!var->name) {
-	    free(var);
+	    sys->free(sys, var);
 	    return ENOMEM;
 	}
 	var->next = NULL;
@@ -275,7 +274,7 @@ isquote(char c)
 }
 
 int
-get_delim_str(char **rtokptr, char **rval, const char **err)
+get_delim_str(sys_data_t *sys, char **rtokptr, char **rval, const char **err)
 {
     char *tokptr = *rtokptr;
     char endc;
@@ -304,7 +303,7 @@ get_delim_str(char **rtokptr, char **rval, const char **err)
 	    val = find_variable(val);
 	    if (!val) {
 		if (rv)
-		    free(rv);
+		    sys->free(sys, rv);
 		*err = "unable to find variable";
 		return -1;
 	    }
@@ -316,7 +315,7 @@ get_delim_str(char **rtokptr, char **rval, const char **err)
 	    while (*tokptr != endc) {
 		if (*tokptr == '\0') {
 		    if (rv)
-			free(rv);
+			sys->free(sys, rv);
 		    *err = "End of line in string";
 		    return -1;
 		}
@@ -326,24 +325,24 @@ get_delim_str(char **rtokptr, char **rval, const char **err)
 	    tokptr++;
 	} else {
 	    if (rv)
-		free(rv);
+		sys->free(sys, rv);
 	    *err = "string value must start with '\"' or '''";
 	    return -1;
 	}
 
 	if (rv) {
-	    char *newrv = malloc(strlen(rv) + strlen(val) + 1);
+	    char *newrv = sys->alloc(sys, strlen(rv) + strlen(val) + 1);
 	    if (!newrv) {
-		free(rv);
+		sys->free(sys, rv);
 		*err = "Out of memory copying string";
 		return -1;
 	    }
 	    strcpy(newrv, rv);
 	    strcat(newrv, val);
-	    free(rv);
+	    sys->free(sys, rv);
 	    rv = newrv;
 	} else {
-	    rv = strdup(val);
+	    rv = sys_strdup(sys, val);
 	    if (!rv) {
 		*err = "Out of memory copying string";
 		return -1;
@@ -637,6 +636,11 @@ get_user(char **tokptr, sys_data_t *sys, const char **err)
 	return -1;
     }
 
+    if (num == 0) {
+	*err = "User number 0 is invalid";
+	return -1;
+    }
+
     rv = get_bool(tokptr, &val, err);
     if (rv)
 	return rv;
@@ -771,12 +775,12 @@ read_config(sys_data_t *sys,
 		errstr = "No variable supplied for define";
 		goto next;
 	    }
-	    err = get_delim_str(&tokptr, &value, &errstr);
+	    err = get_delim_str(sys, &tokptr, &value, &errstr);
 	    if (err)
 		goto next;
-	    err = add_variable(varname, value);
+	    err = add_variable(sys, varname, value);
 	    if (err) {
-		free(value);
+		sys->free(sys, value);
 		err = ENOMEM;
 		errstr = "Out of memory";
 		goto next;
@@ -785,11 +789,11 @@ read_config(sys_data_t *sys,
 	    char *library = NULL, *initstr = NULL;
 	    struct dliblist *dlib, *dlibp;
 
-	    err = get_delim_str(&tokptr, &library, &errstr);
+	    err = get_delim_str(sys, &tokptr, &library, &errstr);
 	    if (!err)
-		err = get_delim_str(&tokptr, &initstr, &errstr);
+		err = get_delim_str(sys, &tokptr, &initstr, &errstr);
 	    if (!err) {
-		dlib = malloc(sizeof(*dlib));
+		dlib = sys->alloc(sys, sizeof(*dlib));
 		if (!dlib) {
 		    err = ENOMEM;
 		    errstr = "Out of memory";
@@ -809,9 +813,9 @@ read_config(sys_data_t *sys,
 	    }
 	    if (err) {
 		if (library)
-		    free((char *) library);
+		    sys->free(sys, (char *) library);
 		if (initstr)
-		    free((char *) initstr);
+		    sys->free(sys, (char *) initstr);
 	    }
 	    goto next;
 	}
@@ -835,17 +839,18 @@ read_config(sys_data_t *sys,
 	} else if (strcmp(tok, "serial") == 0) {
 	    err = serserv_read_config(&tokptr, sys, &errstr);
 	} else if (strcmp(tok, "sol") == 0) {
-	    err = sol_read_config(&tokptr, sys, &errstr);
+	    err = sys->sol_read_config(&tokptr, sys, &errstr);
 	} else if (strcmp(tok, "chassis_control") == 0) {
 	    char *prog;
-	    err = get_delim_str(&tokptr, &prog, &errstr);
+	    err = get_delim_str(sys, &tokptr, &prog, &errstr);
 	    if (!err)
-		ipmi_set_chassis_control_prog(sys->mc, prog);
+		sys->set_chassis_control_prog(sys->mc, prog);
 	} else if (strcmp(tok, "name") == 0) {
-	    err = get_delim_str(&tokptr, &sys->name, &errstr);
+	    err = get_delim_str(sys, &tokptr, &sys->name, &errstr);
 	} else if (strcmp(tok, "startcmd") == 0) {
 	    if (sys->startcmd)
-		err = get_delim_str(&tokptr, &sys->startcmd->startcmd, &errstr);
+		err = get_delim_str(sys, &tokptr, &sys->startcmd->startcmd,
+				    &errstr);
 	} else if (strcmp(tok, "startnow") == 0) {
 	    if (sys->startcmd)
 		err = get_bool(&tokptr, &sys->startcmd->startnow, &errstr);
@@ -861,7 +866,7 @@ read_config(sys_data_t *sys,
 	    err = get_uchar(&tokptr, &ipmb, &errstr);
 	    if (!err) {
 		lmc_data_t *mc;
-		err = ipmi_mc_alloc_unconfigured(sys, ipmb, &mc);
+		err = sys->mc_alloc_unconfigured(sys, ipmb, &mc);
 		if (err == ENOMEM) {
 		    errstr = "Out of memory";
 		    err = -1;
@@ -870,11 +875,11 @@ read_config(sys_data_t *sys,
 		    err = -1;
 		} else {
 		    sys->mc = mc;
-		    sys->cusers = ipmi_mc_get_users(mc);
-		    sys->chan_set = ipmi_mc_get_channelset(mc);
-		    sys->cpef = ipmi_mc_get_pef(mc);
-		    sys->startcmd = ipmi_mc_get_startcmdinfo(mc);
-		    sys->sol = ipmi_mc_get_sol(mc);
+		    sys->cusers = sys->mc_get_users(mc);
+		    sys->chan_set = sys->mc_get_channelset(mc);
+		    sys->cpef = sys->mc_get_pef(mc);
+		    sys->startcmd = sys->mc_get_startcmdinfo(mc);
+		    sys->sol = sys->mc_get_sol(mc);
 		}
 	    }
 	} else if (strcmp(tok, "console") == 0) {
