@@ -260,7 +260,7 @@ read_command_file(emu_out_t *out, emu_data_t *emu, const char *command_file)
 	char *buffer;
 	int  pos = 0;
 
-	buffer = malloc(INPUT_BUFFER_SIZE);
+	buffer = emu->sys->alloc(emu->sys, INPUT_BUFFER_SIZE);
 	if (!buffer) {
 	    out->eprintf(out, "Could not allocate buffer memory\n");
 	    rv = ENOMEM;
@@ -293,7 +293,7 @@ read_command_file(emu_out_t *out, emu_data_t *emu, const char *command_file)
 	}
  out:
 	if (buffer)
-	    free(buffer);
+	    emu->sys->free(emu->sys, buffer);
 	fclose(f);
     }
 
@@ -929,16 +929,16 @@ mc_add_fru_data(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	if (rv)
 	    return rv;
 
-	rv = get_delim_str(toks, &frufn, &errstr);
+	rv = get_delim_str(emu->sys, toks, &frufn, &errstr);
 	if (rv) {
-	    out->eprintf(out, "**Error with FRU filename: %d", strerror(rv));
+	    out->eprintf(out, "**Error with FRU filename: %s", errstr);
 	    return rv;
 	}
 	rv = ipmi_mc_add_fru_file(mc, devid, length, file_offset,
 				  (void *) frufn);
 	if (rv)
 	    out->eprintf(out, "**Unable to add FRU file, error 0x%x\n", rv);
-	free(frufn);
+	emu->sys->free(emu->sys, frufn);
     } else if (strcmp(tok, "data") == 0) {
 	for (i=0; i<length; i++) {
 	    rv = emu_get_uchar(out, toks, &data[i], "data byte", 1);
@@ -987,7 +987,7 @@ mc_dump_fru_data(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	goto out;
     }
 
-    data = malloc(length);
+    data = emu->sys->alloc(emu->sys, length);
     if (!data) {
 	out->eprintf(out, "**Unable to dump FRU data, out of memory\n", rv);
 	goto out;
@@ -1008,7 +1008,7 @@ mc_dump_fru_data(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
  out:
     if (data)
-	free(data);
+	emu->sys->free(emu->sys, data);
     return rv;
 }
 
@@ -1085,7 +1085,7 @@ read_cmds(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
     const char *errstr;
     int err;
 
-    err = get_delim_str(toks, &filename, &errstr);
+    err = get_delim_str(emu->sys, toks, &filename, &errstr);
     if (err) {
 	out->eprintf(out, "Could not get include filename: %s\n", errstr);
 	return err;
@@ -1097,7 +1097,8 @@ read_cmds(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	strncmp(filename, "./", 2) &&
 	strncmp(filename, "../", 3))
     {
-	char *nf = malloc(strlen(BASE_CONF_STR) + strlen(filename) + 2);
+	char *nf = emu->sys->alloc(emu->sys,
+				   strlen(BASE_CONF_STR) + strlen(filename) + 2);
 	if (!nf) {
 	    out->eprintf(out, "Out of memory in include\n", errstr);
 	    goto out_err;
@@ -1105,7 +1106,7 @@ read_cmds(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	strcpy(nf, BASE_CONF_STR);
 	strcat(nf, "/");
 	strcat(nf, filename);
-	free((char *) filename);
+	emu->sys->free(emu->sys, (char *) filename);
 	filename = nf;
 	err = read_command_file(out, emu, filename);
 	if (err) {
@@ -1114,7 +1115,7 @@ read_cmds(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
     }
 
   out_err:
-    free((char *) filename);
+    emu->sys->free(emu->sys, (char *) filename);
     return err;
 }
 
@@ -1199,14 +1200,14 @@ do_define(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	out->eprintf(out, "No variable name given for define\n");
 	return EINVAL;
     }
-    err = get_delim_str(toks, &value, &errstr);
+    err = get_delim_str(emu->sys, toks, &value, &errstr);
     if (err) {
 	out->eprintf(out, "Could not get variable %s value: %s\n", name, errstr);
 	return err;
     }
-    err = add_variable(name, value);
+    err = add_variable(emu->sys, name, value);
     if (err) {
-	free(value);
+	emu->sys->free(emu->sys, value);
 	out->eprintf(out, "Out of memory setting variable %s\n", name);
 	return err;
     }
@@ -1258,17 +1259,17 @@ static struct emu_cmd_info cmds[] =
 static struct emu_cmd_info *cmdlist = &cmds[0];
 
 int
-ipmi_emu_add_cmd(const char *name, unsigned int flags,
+ipmi_emu_add_cmd(emu_data_t *emu, const char *name, unsigned int flags,
 		 ipmi_emu_cmd_handler handler)
 {
     struct emu_cmd_info *mcmd;
 
-    mcmd = malloc(sizeof(*mcmd));
+    mcmd = emu->sys->alloc(emu->sys, sizeof(*mcmd));
     if (!mcmd)
 	return ENOMEM;
-    mcmd->name = strdup(name);
+    mcmd->name = sys_strdup(emu->sys, name);
     if (!mcmd->name) {
-	free(mcmd);
+	emu->sys->free(emu->sys, mcmd);
 	return ENOMEM;
     }
     mcmd->flags = flags;
@@ -1319,3 +1320,9 @@ ipmi_emu_cmd(emu_out_t *out, emu_data_t *emu, char *cmd_str)
  out:
     return rv;
 }
+
+sys_data_t *
+ipmi_emu_get_sysconfig(emu_data_t *emu)
+{
+    return emu->sys;
+}
