@@ -87,13 +87,14 @@
 #include <OpenIPMI/ipmi_mc.h>
 #include <OpenIPMI/os_handler.h>
 #include <OpenIPMI/ipmi_posix.h>
-#include <OpenIPMI/serv.h>
-#include <OpenIPMI/lanserv.h>
-#include <OpenIPMI/serserv.h>
-#include <OpenIPMI/ipmbserv.h>
 
 #include "emu.h"
+#include "bmc.h"
+#include <OpenIPMI/serserv.h>
+#include <OpenIPMI/ipmbserv.h>
 #include <OpenIPMI/persist.h>
+#include <OpenIPMI/internal/winsock_compat.h>
+#include "ipmi_sim.h"
 
 #define MAX_ADDR 4
 
@@ -131,11 +132,13 @@ static void isim_add_fd(int fd)
 
 static void isim_close_fds(void)
 {
-    isim_fd_t *n = isim_fds;
+    isim_fd_t *n;
 
-    while(n) {
+    while(isim_fds) {
+	n = isim_fds;
+	isim_fds = n->next;
 	close(n->fd);
-	n = n->next;
+	free(n);
     }
 }
 
@@ -175,7 +178,10 @@ static misc_data_t *global_misc_data;
 static void *
 balloc(sys_data_t *sys, int size)
 {
-    return malloc(size);
+    void *rv = malloc(size);
+    if (rv)
+	memset(rv, 0, size);
+    return rv;
 }
 
 static void
@@ -186,54 +192,28 @@ bfree(sys_data_t *sys, void *data)
 
 typedef struct sim_addr_s
 {
-    struct sockaddr_storage addr;
-    socklen_t       addr_len;
     int             xmit_fd;
+    socklen_t       addr_len;
+    /* struct sockaddr_storage addr; */
 } sim_addr_t;
 
 static int
 smi_send(channel_t *chan, msg_t *msg)
 {
-    misc_data_t      *data = chan->oem.user_data;
-    unsigned char    msgd[36];
-    unsigned int     msgd_len = sizeof(msgd);
-
-    ipmi_emu_handle_msg(data->emu, chan->mc, msg, msgd, &msgd_len);
-
-    ipmi_handle_smi_rsp(chan, msg, msgd, msgd_len);
+    ipmi_mc_handle_msg(chan->mc, msg);
     return 0;
 }
 
 static int
 gen_rand(lanserv_data_t *lan, void *data, int len)
 {
-    int fd = open("/dev/urandom", O_RDONLY);
-    int rv;
-
-    if (fd == -1)
-	return errno;
-
-    while (len > 0) {
-	rv = read(fd, data, len);
-	if (rv < 0) {
-	    rv = errno;
-	    goto out;
-	}
-	len -= rv;
-    }
-
-    rv = 0;
-
- out:
-    close(fd);
-    return rv;
+    return gen_random(data, len);
 }
 
 static int
 sys_gen_rand(sys_data_t *lan, void *data, int len)
 {
-    gen_rand(NULL, data, len);
-    return 0;
+    return gen_random(data, len);
 }
 
 static void
@@ -251,7 +231,8 @@ lan_send(lanserv_data_t *lan,
 	return;
 
     memset(&msg, 0, sizeof(msg));
-    msg.msg_name = &(l->addr);
+    /* Address is after the sim_addr_t struct. */
+    msg.msg_name = ((uint8_t *) l) + sizeof(l);
     msg.msg_namelen = l->addr_len;
     msg.msg_iov = data;
     msg.msg_iovlen = vecs;
@@ -265,14 +246,15 @@ lan_send(lanserv_data_t *lan,
 static void
 lan_data_ready(int lan_fd, void *cb_data, os_hnd_fd_id_t *id)
 {
-    lanserv_data_t    *lan = cb_data;
-    int           len;
-    sim_addr_t    l;
-    unsigned char msgd[256];
+    lanserv_data_t *lan = cb_data;
+    int len;
+    uint8_t addrdata[sizeof(sim_addr_t) + sizeof(struct sockaddr_storage)];
+    sim_addr_t *l = (sim_addr_t *) addrdata;
+    uint8_t msgd[256];
+    struct sockaddr *addr = (struct sockaddr *) (((char *) l) + sizeof(*l));
 
-    l.addr_len = sizeof(l.addr);
-    len = recvfrom(lan_fd, msgd, sizeof(msgd), 0,
-		   (struct sockaddr *) &(l.addr), &(l.addr_len));
+    l->addr_len = sizeof(struct sockaddr_storage);
+    len = recvfrom(lan_fd, msgd, sizeof(msgd), 0, addr, &l->addr_len);
     if (len < 0) {
 	if (errno != EINTR) {
 	    perror("Error receiving message");
@@ -280,13 +262,12 @@ lan_data_ready(int lan_fd, void *cb_data, os_hnd_fd_id_t *id)
 	}
 	goto out;
     }
-    l.xmit_fd = lan_fd;
+    l->xmit_fd = lan_fd;
 
-    if (lan->sysinfo->debug & DEBUG_RAW_MSG) {
-	debug_log_raw_msg(lan->sysinfo, (void *) &l.addr, l.addr_len,
+    if (lan->sys->debug & DEBUG_RAW_MSG) {
+	debug_log_raw_msg(lan->sys, (uint8_t *) addr, l->addr_len,
 			  "Raw LAN receive from:");
-	debug_log_raw_msg(lan->sysinfo, msgd, len,
-			  " Receive message:");
+	debug_log_raw_msg(lan->sys, msgd, len, " Receive message:");
     }
 
     if (len < 4)
@@ -298,23 +279,59 @@ lan_data_ready(int lan_fd, void *cb_data, os_hnd_fd_id_t *id)
     /* Check the message class. */
     switch (msgd[3]) {
 	case 6:
-	    handle_asf(lan, msgd, len, &l, sizeof(l));
+	    handle_asf(lan, msgd, len, l, sizeof(*l) + l->addr_len);
 	    break;
 
 	case 7:
-	    ipmi_handle_lan_msg(lan, msgd, len, &l, sizeof(l));
+	    ipmi_handle_lan_msg(lan, msgd, len, l, sizeof(*l) + l->addr_len);
 	    break;
     }
  out:
     return;
 }
 
+/*
+ * For assigning zero ports.
+ */
+#define IP_DYNRANGE_START	49152
+#define IP_DYNRANGE_END		65535
+
 static int
-open_lan_fd(struct sockaddr *addr, socklen_t addr_len)
+open_lan_fd(struct sockaddr *addr, socklen_t addr_len, unsigned short *rport)
 {
     int fd;
     int rv;
-    int opt;
+    int port, curr_port = IP_DYNRANGE_START;
+    struct sockaddr_storage sas;
+    struct sockaddr *sa;
+    socklen_t addrlen;
+
+    if (addr_len > sizeof(sas)) {
+	fprintf(stderr, "Address too large: %ld", (long) addr_len);
+	exit(1);
+    }
+    memcpy(&sas, addr, addr_len);
+
+    sa = (struct sockaddr *) &sas;
+    switch (sa->sa_family) {
+    case AF_INET: {
+	struct sockaddr_in *sai = (struct sockaddr_in *) sa;
+
+	port = ntohs(sai->sin_port);
+	break;
+    }
+
+    case AF_INET6: {
+	struct sockaddr_in6 *sai6 = (struct sockaddr_in6 *) sa;
+
+	port = ntohs(sai6->sin6_port);
+	break;
+    }
+
+    default:
+	fprintf(stderr, "Invalid socket address family: %d\n", sa->sa_family);
+	exit(1);
+    }
 
     fd = socket(addr->sa_family, SOCK_DGRAM, IPPROTO_UDP);
     if (fd == -1) {
@@ -322,21 +339,76 @@ open_lan_fd(struct sockaddr *addr, socklen_t addr_len)
 	exit(1);
     }
 
-    opt = 1;
-    rv = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
+    /* Don't set reuseaddr for UDP, you can get a shared port. */
+ retry:
+    if (port == 0) {
+	printf("Trying port %d\n", curr_port);
+	switch (sa->sa_family) {
+	case AF_INET: {
+	    struct sockaddr_in *sai = (struct sockaddr_in *) sa;
+
+	    sai->sin_port = htons(curr_port);
+	    break;
+	}
+
+	case AF_INET6: {
+	    struct sockaddr_in6 *sai6 = (struct sockaddr_in6 *) sa;
+
+	    sai6->sin6_port = htons(curr_port);
+	    break;
+	}
+
+	default:
+	    close(fd);
+	    fprintf(stderr, "Invalid socket address family: %d\n",
+		    sa->sa_family);
+	    exit(1);
+	}
+    }
+
+    rv = bind(fd, sa, addr_len);
     if (rv == -1) {
-	fprintf(stderr, "Unable to set SO_REUSEADDR: %s\n",
+	if (port == 0) {
+	    curr_port++;
+	    if (curr_port <= IP_DYNRANGE_END)
+		goto retry;
+	}
+	fprintf(stderr, "Unable to bind to LAN port: %s\n",
 		strerror(errno));
 	exit(1);
     }
 
-    rv = bind(fd, addr, addr_len);
-    if (rv == -1) {
-	fprintf(stderr, "Unable to bind to LAN port: %s\n",
+    addrlen = sizeof(sas);
+    rv = getsockname(fd, (struct sockaddr *) &sas, &addrlen);
+    if (rv) {
+	close(fd);
+	fprintf(stderr, "Unable to get socket address: %s\n",
 		strerror(errno));
 	exit(1);
     }
 
+    sa = (struct sockaddr *) &sas;
+    switch (sa->sa_family) {
+    case AF_INET: {
+	struct sockaddr_in *sai = (struct sockaddr_in *) sa;
+
+	*rport = ntohs(sai->sin_port);
+	break;
+    }
+
+    case AF_INET6: {
+	struct sockaddr_in6 *sai6 = (struct sockaddr_in6 *) sa;
+
+	*rport = ntohs(sai6->sin6_port);
+	break;
+    }
+
+    default:
+	close(fd);
+	fprintf(stderr, "Invalid socket address family: %d\n", sa->sa_family);
+	exit(1);
+    }
+
     isim_add_fd(fd);
 
     return fd;
@@ -370,7 +442,7 @@ lan_channel_init(void *info, channel_t *chan)
 
     if (lan->lan_addr_set) {
 	lan_fd = open_lan_fd(&lan->lan_addr.addr.s_ipsock.s_addr0,
-			     lan->lan_addr.addr_len);
+			     lan->lan_addr.addr_len, &lan->port);
 	if (lan_fd == -1) {
 	    fprintf(stderr, "Unable to open LAN address\n");
 	    exit(1);
@@ -390,6 +462,8 @@ lan_channel_init(void *info, channel_t *chan)
 	    fprintf(stderr, "Unable to add socket wait: 0x%x\n", err);
 	    exit(1);
 	}
+	printf("Opened UDP port %d\n\r", lan->port);
+	fflush(stdout);
     }
 
     return err;
@@ -417,7 +491,7 @@ ser_data_ready(int fd, void *cb_data, os_hnd_fd_id_t *id)
     int           len;
     unsigned char msgd[256];
 
-    len = read(fd, msgd, sizeof(msgd));
+    len = recv(fd, msgd, sizeof(msgd), 0);
     if (len <= 0) {
 	if ((len < 0) && (errno == EINTR))
 	    return;
@@ -425,7 +499,7 @@ ser_data_ready(int fd, void *cb_data, os_hnd_fd_id_t *id)
 	if (ser->codec->disconnected)
 	    ser->codec->disconnected(ser);
 	ser->os_hnd->remove_fd_to_wait_for(ser->os_hnd, id);
-	close(fd);
+	close_socket(fd);
 	ser->con_fd = -1;
 	return;
     }
@@ -451,7 +525,7 @@ ser_bind_ready(int fd, void *cb_data, os_hnd_fd_id_t *id)
     }
 
     if (ser->con_fd >= 0) {
-	close(rv);
+	close_socket(rv);
 	return;
     }
 
@@ -466,7 +540,7 @@ ser_bind_ready(int fd, void *cb_data, os_hnd_fd_id_t *id)
     if (err) {
 	fprintf(stderr, "Unable to add serial socket wait: 0x%x\n", err);
 	ser->con_fd = -1;
-	close(rv);
+	close_socket(rv);
     } else {
 	if (ser->codec->connected)
 	    ser->codec->connected(ser);
@@ -589,13 +663,13 @@ static void
 ipmb_data_ready(int fd, void *cb_data, os_hnd_fd_id_t *id)
 {
     ipmbserv_data_t *ipmb = cb_data;
-    unsigned int  len;
+    ssize_t len;
     unsigned char msgd[256];
 
     len = read(fd, msgd, sizeof(msgd));
 
     if (ipmb->sysinfo->debug & DEBUG_MSG)
-        printf(">ipmb_data_ready size %d\n", len);
+        printf(">ipmb_data_ready size %ld\n", (long) len);
     if (len <= 0) {
         if ((len < 0) && (errno == EINTR))
             return;
@@ -675,12 +749,12 @@ isim_log(sys_data_t *sys, int logtype, msg_t *msg, const char *format,
 	int pos;
 	unsigned int i;
 
-#define mformat " channel=%d netfn=0x%x cmd=0x%x rs_addr=0x%x rs_lun=0x%x" \
-	    " rq_addr=0x%x\n rq_lun=0x%x rq_seq=0x%x\n"
+#define mformat " channel=%d netfn=0x%x cmd=0x%x daddr=0x%x dlun=0x%x" \
+	    " saddr=0x%x\n slun=0x%x rq_seq=0x%x\n"
 
 	len += snprintf(&dummy, 0, mformat, msg->channel, msg->netfn,
-			msg->cmd, msg->rs_addr, msg->rs_lun, msg->rq_addr,
-			msg->rq_lun, msg->rq_seq);
+			msg->cmd, msg->daddr, msg->dlun, msg->saddr,
+			msg->slun, msg->rq_seq);
 	len += 3 * msg->len + 3;
 	str = malloc(len);
 	if (!str)
@@ -688,7 +762,7 @@ isim_log(sys_data_t *sys, int logtype, msg_t *msg, const char *format,
 	pos = vsprintf(str, format, ap);
 	str[pos++] = '\n';
 	pos += sprintf(str + pos, mformat, msg->channel, msg->netfn, msg->cmd,
-		       msg->rs_addr, msg->rs_lun, msg->rq_addr, msg->rq_lun,
+		       msg->daddr, msg->dlun, msg->saddr, msg->slun,
 		       msg->rq_seq);
 #undef mformat
 	for (i = 0; i < msg->len; i++)
@@ -730,21 +804,6 @@ sim_log(sys_data_t *sys, int logtype, msg_t *msg, const char *format, ...)
     va_end(ap);
 }
 
-static void
-sim_chan_log(channel_t *chan, int logtype, msg_t *msg, const char *format, ...)
-{
-    va_list ap;
-    char dummy;
-    int len;
-
-    va_start(ap, format);
-    len = vsnprintf(&dummy, 1, format, ap);
-    va_end(ap);
-    va_start(ap, format);
-    isim_log(global_misc_data->sys, logtype, msg, format, ap, len);
-    va_end(ap);
-}
-
 static struct poptOption poptOpts[]=
 {
     {
@@ -1049,7 +1108,7 @@ console_bind_ready(int fd, void *cb_data, os_hnd_fd_id_t *id)
     if (!newcon) {
 	char *msg = "Out of memory\n";
 	err = write(rv, msg, strlen(msg));
-	close(rv);
+	close_socket(rv);
 	return;
     }
 
@@ -1072,7 +1131,7 @@ console_bind_ready(int fd, void *cb_data, os_hnd_fd_id_t *id)
     if (err) {
 	char *msg = "Unable to add socket wait\n";
 	err = write(rv, msg, strlen(msg));
-	close(rv);
+	close_socket(rv);
 	free(newcon);
 	return;
     }
@@ -1291,8 +1350,8 @@ ipmi_free_timer(ipmi_timer_t *timer)
 
 static ipmi_tick_handler_t *tick_handlers;
 
-void
-ipmi_register_tick_handler(ipmi_tick_handler_t *handler)
+static void
+is_register_tick_handler(ipmi_tick_handler_t *handler)
 {
     handler->next = tick_handlers;
     tick_handlers = handler;
@@ -1323,18 +1382,6 @@ tick(void *cb_data, os_hnd_timer_id_t *id)
     }
 }
 
-static void *
-ialloc(channel_t *chan, int size)
-{
-    return malloc(size);
-}
-
-static void
-ifree(channel_t *chan, void *data)
-{
-    return free(data);
-}
-
 static int sigpipeh[2] = {-1, -1};
 
 static void
@@ -1487,7 +1534,8 @@ main(int argc, const char *argv[])
     }
     poptFreeContext(poptCtx);
 
-    printf("IPMI Simulator version %s\n", PVERSION);
+    printf("IPMI Simulator version %s\n\r", PVERSION);
+    fflush(stdout);
 
     global_misc_data = &data;
 
@@ -1528,12 +1576,22 @@ main(int argc, const char *argv[])
     sysinfo.debug = debug;
     sysinfo.log = sim_log;
     sysinfo.csmi_send = smi_send;
-    sysinfo.clog = sim_chan_log;
-    sysinfo.calloc = ialloc;
-    sysinfo.cfree = ifree;
     sysinfo.lan_channel_init = lan_channel_init;
     sysinfo.ser_channel_init = ser_channel_init;
     sysinfo.ipmb_channel_init = ipmb_channel_init;
+    sysinfo.mc_alloc_unconfigured = is_mc_alloc_unconfigured;
+    sysinfo.resend_atn = is_resend_atn;
+    sysinfo.mc_get_ipmb = is_mc_get_ipmb;
+    sysinfo.mc_get_channelset = is_mc_get_channelset;
+    sysinfo.mc_get_sol = is_mc_get_sol;
+    sysinfo.mc_get_startcmdinfo = is_mc_get_startcmdinfo;
+    sysinfo.mc_get_users = is_mc_get_users;
+    sysinfo.mc_users_changed = is_mc_users_changed;
+    sysinfo.mc_get_pef = is_mc_get_pef;
+    sysinfo.sol_read_config = is_sol_read_config;
+    sysinfo.set_chassis_control_prog = is_set_chassis_control_prog;
+    sysinfo.register_tick_handler = is_register_tick_handler;
+
     data.sys = &sysinfo;
 
     err = pipe(sigpipeh);
@@ -1581,18 +1639,18 @@ main(int argc, const char *argv[])
     stdio_console.prev = NULL;
     data.consoles = &stdio_console;
 
-    err = ipmi_mc_alloc_unconfigured(&sysinfo, 0x20, &mc);
+    err = is_mc_alloc_unconfigured(&sysinfo, 0x20, &mc);
     if (err) {
 	if (err == ENOMEM)
 	    fprintf(stderr, "Out of memory allocation BMC MC\n");
 	exit(1);
     }
     sysinfo.mc = mc;
-    sysinfo.chan_set = ipmi_mc_get_channelset(mc);
-    sysinfo.startcmd = ipmi_mc_get_startcmdinfo(mc);
-    sysinfo.cpef = ipmi_mc_get_pef(mc);
-    sysinfo.cusers = ipmi_mc_get_users(mc);
-    sysinfo.sol = ipmi_mc_get_sol(mc);
+    sysinfo.chan_set = is_mc_get_channelset(mc);
+    sysinfo.startcmd = is_mc_get_startcmdinfo(mc);
+    sysinfo.cpef = is_mc_get_pef(mc);
+    sysinfo.cusers = is_mc_get_users(mc);
+    sysinfo.sol = is_mc_get_sol(mc);
 
     if (read_config(&sysinfo, config_file, print_version))
 	exit(1);
@@ -1605,7 +1663,7 @@ main(int argc, const char *argv[])
 	exit(1);
     }
 
-    err = persist_init("ipmi_sim", sysinfo.name, statedir);
+    err = persist_init(&sysinfo, "ipmi_sim", sysinfo.name, statedir);
     if (err) {
 	fprintf(stderr, "Unable to initialize persistence: %s\n",
 		strerror(err));
@@ -1614,7 +1672,7 @@ main(int argc, const char *argv[])
 
     read_persist_users(&sysinfo);
 
-    err = sol_init(&sysinfo);
+    err = sol_init(data.emu);
     if (err) {
 	fprintf(stderr, "Unable to initialize SOL: %s\n",
 		strerror(err));
