@@ -72,8 +72,8 @@
 static void
 raw_send(serserv_data_t *si, unsigned char *data, unsigned int len)
 {
-    if (si->sysinfo->debug & DEBUG_RAW_MSG)
-	debug_log_raw_msg(si->sysinfo, data, len, "Raw serial send:");
+    if (si->sys->debug & DEBUG_RAW_MSG)
+	debug_log_raw_msg(si->sys, data, len, "Raw serial send:");
     si->send_out(si, data, len);
 }
 
@@ -107,12 +107,12 @@ unformat_ipmb_msg(msg_t *msg, unsigned char *msgd, unsigned int len,
     }
     len--;
 
-    msg->rs_addr = msgd[0];
+    msg->daddr = msgd[0];
+    msg->dlun = msgd[1] & 3;
     msg->netfn = msgd[1] >> 2;
-    msg->rs_lun = msgd[1] & 3;
-    msg->rq_addr = msgd[3];
+    msg->saddr = msgd[3];
+    msg->slun = msgd[4] & 3;
     msg->rq_seq = msgd[4] >> 2;
-    msg->rq_lun = msgd[4] & 3;
     msg->cmd = msgd[5];
 
     msg->len = len - 6;
@@ -128,11 +128,11 @@ static void
 format_ipmb_rsp(msg_t *msg, unsigned char *msgd,
 		unsigned int *msgd_len, serserv_data_t *mi)
 {
-    msgd[0] = msg->rs_addr;
-    msgd[1] = (msg->netfn << 2) | msg->rs_lun;
+    msgd[0] = msg->daddr;
+    msgd[1] = (msg->netfn << 2) | msg->dlun;
     msgd[2] = -ipmb_checksum(msgd, 2, 0);
-    msgd[3] = msg->rq_addr;
-    msgd[4] = (msg->rq_seq << 2) | msg->rq_lun;
+    msgd[3] = msg->saddr;
+    msgd[4] = (msg->rq_seq << 2) | msg->slun;
     msgd[5] = msg->cmd;
     memcpy(msgd + 6, msg->data, msg->len);
     *msgd_len = msg->len + 6;
@@ -188,7 +188,7 @@ ra_ipmb_handler(channel_t *chan, msg_t *msg)
     serserv_data_t *si = chan->chan_info;
 
     ra_format_msg(msg->data, msg->len, si);
-    free(msg);
+    si->sys->free(si->sys, msg);
     return 1;
 }
 
@@ -205,8 +205,8 @@ static int ra_unformat_msg(unsigned char *r, unsigned int len,
     unsigned int i = 0;
     int          rv;
 
-    if (si->sysinfo->debug & DEBUG_RAW_MSG)
-	debug_log_raw_msg(si->sysinfo, r, len, "Raw serial receive:");
+    if (si->sys->debug & DEBUG_RAW_MSG)
+	debug_log_raw_msg(si->sys, r, len, "Raw serial receive:");
 
     while (p < len) {
 	rv = fromhex(r[p]);
@@ -228,17 +228,17 @@ static int ra_unformat_msg(unsigned char *r, unsigned int len,
 	return -1;
 
     memset(&msg, 0, sizeof(msg));
-    if ((o[0] == si->sysinfo->bmc_ipmb) || (o[0] == 1)) {
+    if ((o[0] == si->sys->bmc_ipmb) || (o[0] == 1)) {
 	rv = unformat_ipmb_msg(&msg, o, i, si);
 	if (rv)
 	    return rv;
     } else {
-	msg.rs_addr = 1;
+	msg.daddr = 1;
+	msg.dlun = 0;
 	msg.netfn = IPMI_APP_NETFN;
-	msg.rs_lun = 0;
-	msg.rq_addr = 1;
+	msg.saddr = 1;
+	msg.slun = 0;
 	msg.rq_seq = 0;
-	msg.rq_lun = 0;
 	msg.cmd = IPMI_SEND_MSG_CMD;
 
 	msg.len = i + 1;
@@ -310,7 +310,7 @@ int
 ra_setup(serserv_data_t *si)
 {
     struct ra_data *info;
-    info = malloc(sizeof(*info));
+    info = si->sys->alloc(si->sys, sizeof(*info));
     if (!info)
 	return -1;
 
@@ -346,8 +346,8 @@ dm_handle_msg(unsigned char *imsg, unsigned int len, serserv_data_t *si)
     int rv;
     msg_t msg;
 
-    if (si->sysinfo->debug & DEBUG_RAW_MSG)
-	debug_log_raw_msg(si->sysinfo, imsg, len, "Raw serial receive:");
+    if (si->sys->debug & DEBUG_RAW_MSG)
+	debug_log_raw_msg(si->sys, imsg, len, "Raw serial receive:");
 
     memset(&msg, 0, sizeof(msg));
     rv = unformat_ipmb_msg(&msg, imsg, len, si);
@@ -488,10 +488,9 @@ dm_setup(serserv_data_t *si)
 {
     struct dm_data *info;
 
-    info = malloc(sizeof(*info));
+    info = si->sys->alloc(si->sys, sizeof(*info));
     if (!info)
 	return -1;
-    memset(info, 0, sizeof(*info));
     si->channel.set_atn = handle_attn;
     si->codec_info = info;
     return 0;
@@ -524,7 +523,7 @@ tm_send(msg_t *msg, serserv_data_t *si)
     c[len] = '[';
     len++;
 
-    t = msg->netfn << 2 | msg->rs_lun;
+    t = msg->netfn << 2 | msg->dlun;
     c[len] = hex2char[t >> 4];
     len++;
     c[len] = hex2char[t & 0xf];
@@ -579,8 +578,8 @@ static int tm_unformat_msg(unsigned char *r, unsigned int len,
     unsigned int  i = 0;
     int           rv;
 
-    if (si->sysinfo->debug & DEBUG_RAW_MSG)
-	debug_log_raw_msg(si->sysinfo, r, len, "Raw serial receive:");
+    if (si->sys->debug & DEBUG_RAW_MSG)
+	debug_log_raw_msg(si->sys, r, len, "Raw serial receive:");
 
 #define SKIP_SPACE if (isspace(r[p])) p++
 #define ENSURE_MORE if (p >= len) return -1
@@ -610,7 +609,7 @@ static int tm_unformat_msg(unsigned char *r, unsigned int len,
 
 	memset(&msg, 0, sizeof(msg));
 	msg.netfn = o[0] >> 2;
-	msg.rq_lun = o[0] & 3;
+	msg.dlun = o[0] & 3;
 	msg.rq_seq = o[1] >> 2;
 	msg.cmd = o[2];
 	msg.data = o + 3;
@@ -691,7 +690,7 @@ tm_setup(serserv_data_t *si)
 {
     struct tm_data *info;
 
-    info = malloc(sizeof(*info));
+    info = si->sys->alloc(si->sys, sizeof(*info));
     if (!info)
 	return -1;
 
@@ -761,8 +760,8 @@ vm_handle_msg(unsigned char *imsg, unsigned int len, serserv_data_t *si)
 {
     msg_t msg;
     
-    if (si->sysinfo->debug & DEBUG_RAW_MSG)
-	debug_log_raw_msg(si->sysinfo, imsg, len, "Raw serial receive:");
+    if (si->sys->debug & DEBUG_RAW_MSG)
+	debug_log_raw_msg(si->sys, imsg, len, "Raw serial receive:");
 
     if (len < 4) {
 	fprintf(stderr, "Message too short\n");
@@ -778,7 +777,7 @@ vm_handle_msg(unsigned char *imsg, unsigned int len, serserv_data_t *si)
     memset(&msg, 0, sizeof(msg));
     msg.rq_seq = imsg[0];
     msg.netfn = imsg[1] >> 2;
-    msg.rs_lun = imsg[1] & 0x3;
+    msg.dlun = imsg[1] & 0x3;
     msg.cmd = imsg[2];
     msg.len = len - 3;
     msg.data = imsg + 3;
@@ -791,8 +790,8 @@ vm_handle_cmd(unsigned char *imsg, unsigned int len, serserv_data_t *si)
 {
     struct vm_data *info = si->codec_info;
 
-    if (si->sysinfo->debug & DEBUG_RAW_MSG)
-	debug_log_raw_msg(si->sysinfo, imsg, len, "Raw serial cmd:");
+    if (si->sys->debug & DEBUG_RAW_MSG)
+	debug_log_raw_msg(si->sys, imsg, len, "Raw serial cmd:");
 
     if (len < 1)
 	return;
@@ -821,8 +820,8 @@ vm_handle_cmd(unsigned char *imsg, unsigned int len, serserv_data_t *si)
 
     case VM_CMD_RESET:
 	/* The remote end reset, report it. */
-	if (si->sysinfo->target_reset)
-	    si->sysinfo->target_reset(si->sysinfo);
+	if (si->sys->target_reset)
+	    si->sys->target_reset(si->sys);
 	break;
     }
 }
@@ -905,7 +904,7 @@ vm_send(msg_t *imsg, serserv_data_t *si)
     vm_add_char(ch, c, &len);
     csum = ipmb_checksum(&ch, 1, 0);
 
-    ch = (imsg->netfn << 2) | imsg->rs_lun;
+    ch = (imsg->netfn << 2) | imsg->dlun;
     vm_add_char(ch, c, &len);
     csum = ipmb_checksum(&ch, 1, csum);
 
@@ -1006,8 +1005,8 @@ vm_connected(serserv_data_t *si)
     c[len++] = VM_CMD_CHAR;
     raw_send(si, c, len);
     si->connected = 1;
-    if (si->sysinfo->resend_atn)
-	si->sysinfo->resend_atn(&si->channel);
+    if (si->sys->resend_atn)
+	si->sys->resend_atn(&si->channel);
 }
 
 static void
@@ -1021,10 +1020,9 @@ vm_setup(serserv_data_t *si)
 {
     struct vm_data *info;
 
-    info = malloc(sizeof(*info));
+    info = si->sys->alloc(si->sys, sizeof(*info));
     if (!info)
 	return -1;
-    memset(info, 0, sizeof(*info));
     si->codec_info = info;
     si->channel.hw_op = vm_hw_op;
     si->channel.set_atn = vm_set_attn;
@@ -1175,10 +1173,10 @@ ser_return_rsp(channel_t *chan, msg_t *imsg, rsp_msg_t *rsp)
     msg.cmd = rsp->cmd;
     msg.data = rsp->data;
     msg.len = rsp->data_len;
-    msg.rq_lun = imsg->rs_lun;
-    msg.rq_addr = imsg->rs_addr;
-    msg.rs_lun = imsg->rq_lun;
-    msg.rs_addr = imsg->rq_addr;
+    msg.daddr = imsg->saddr;
+    msg.dlun = imsg->slun;
+    msg.saddr = imsg->daddr;
+    msg.slun = imsg->dlun;
     msg.rq_seq = imsg->rq_seq;
     ser->codec->send(&msg, ser);
 }
@@ -1213,13 +1211,13 @@ serserv_read_config(char **tokptr, sys_data_t *sys, const char **errstr)
     char *endp;
     int err;
     unsigned int chan_num;
+    unsigned int offset = 0;
 
-    ser = malloc(sizeof(*ser));
+    ser = sys->alloc(sys, sizeof(*ser));
     if (!ser) {
 	*errstr = "Out of memory";
 	return -1;
     }
-    memset(ser, 0, sizeof(*ser));
 
     tok = mystrtok(NULL, " \t\n", tokptr);
     if (!tok) {
@@ -1227,28 +1225,47 @@ serserv_read_config(char **tokptr, sys_data_t *sys, const char **errstr)
 	goto out_err;
     }
     ser->channel.session_support = IPMI_CHANNEL_SESSION_LESS;
+    ser->channel.protocol_type = IPMI_CHANNEL_PROTOCOL_TMODE;
     ser->channel.medium_type = IPMI_CHANNEL_MEDIUM_RS232;
-    if (strcmp(tok, "kcs") == 0) {
+    ser->channel.is_serial = 1;
+    if (strncmp(tok, "kcs", 3) == 0) {
 	chan_num = 15;
 	ser->channel.protocol_type = IPMI_CHANNEL_PROTOCOL_KCS;
-    } else if (strcmp(tok, "bt") == 0) {
+	ser->channel.medium_type = IPMI_CHANNEL_MEDIUM_SYS_INTF;
+	offset = 3;
+    } else if (strncmp(tok, "bt", 2) == 0) {
 	chan_num = 15;
 	ser->channel.protocol_type = IPMI_CHANNEL_PROTOCOL_BT_v15;
-    } else if (strcmp(tok, "smic") == 0) {
+	ser->channel.medium_type = IPMI_CHANNEL_MEDIUM_SYS_INTF;
+	offset = 2;
+    } else if (strncmp(tok, "smic", 4) == 0) {
 	chan_num = 15;
 	ser->channel.protocol_type = IPMI_CHANNEL_PROTOCOL_SMIC;
+	ser->channel.medium_type = IPMI_CHANNEL_MEDIUM_SYS_INTF;
+	offset = 4;
+    } else if (strncmp(tok, "ssif", 4) == 0) {
+	chan_num = 15;
+	ser->channel.protocol_type = IPMI_CHANNEL_PROTOCOL_SMBus;
+	ser->channel.medium_type = IPMI_CHANNEL_MEDIUM_SMBUS_v2;
+	offset = 4;
     } else {
 	chan_num = strtoul(tok, &endp, 0);
 	if (*endp != '\0') {
 	    *errstr = "Channel not a valid number";
 	    goto out_err;
 	}
-	ser->channel.protocol_type = IPMI_CHANNEL_PROTOCOL_TMODE;
     }
-    if (chan_num != 15) {
-	*errstr = "Only BMC channel (channel 15, or kcs/bt/smic) is"
-	    " supported for serial";
-	goto out_err;
+    if (offset && tok[offset] != '\0') {
+	if (tok[offset] == '-') {
+	    chan_num = strtoul(tok + offset + 1, &endp, 0);
+	    if (*endp != '\0') {
+		*errstr = "Channel not a valid number";
+		goto out_err;
+	    }
+	} else {
+	    *errstr = "Invalid channel number";
+	    goto out_err;
+	}
     }
 
     if (sys->chan_set[chan_num]) {
@@ -1333,13 +1350,13 @@ serserv_read_config(char **tokptr, sys_data_t *sys, const char **errstr)
 	goto out_err;
     }
 
-    ser->sysinfo = sys;
+    ser->sys = sys;
     ser->channel.chan_info = ser;
 
     sys->chan_set[chan_num] = &ser->channel;
     return 0;
 
  out_err:
-    free(ser);
+    sys->free(sys, ser);
     return -1;
 }
