@@ -351,40 +351,40 @@ ipmi_sel_alloc(ipmi_mc_t       *mc,
 	}
     } else {
 	ipmi_domain_stat_register(domain, "sel_good_scans",
-				  _ipmi_mc_name(mc),
+				  i_ipmi_mc_name(mc),
 				  &sel->sel_good_scans);
 	ipmi_domain_stat_register(domain, "sel_scan_lost_reservation",
-				  _ipmi_mc_name(mc),
+				  i_ipmi_mc_name(mc),
 				  &sel->sel_scan_lost_reservation);
 	ipmi_domain_stat_register(domain, "sel_fail_scan_lost_reservation",
-				  _ipmi_mc_name(mc),
+				  i_ipmi_mc_name(mc),
 				  &sel->sel_fail_scan_lost_reservation);
 	ipmi_domain_stat_register(domain, "sel_received_events",
-				  _ipmi_mc_name(mc),
+				  i_ipmi_mc_name(mc),
 				  &sel->sel_received_events);
 	ipmi_domain_stat_register(domain, "sel_fetch_errors",
-				  _ipmi_mc_name(mc),
+				  i_ipmi_mc_name(mc),
 				  &sel->sel_fetch_errors);
 	ipmi_domain_stat_register(domain, "sel_good_clears",
-				  _ipmi_mc_name(mc),
+				  i_ipmi_mc_name(mc),
 				  &sel->sel_good_clears);
 	ipmi_domain_stat_register(domain, "sel_clear_lost_reservation",
-				  _ipmi_mc_name(mc),
+				  i_ipmi_mc_name(mc),
 				  &sel->sel_clear_lost_reservation);
 	ipmi_domain_stat_register(domain, "sel_clear_errors",
-				  _ipmi_mc_name(mc),
+				  i_ipmi_mc_name(mc),
 				  &sel->sel_clear_errors);
 	ipmi_domain_stat_register(domain, "sel_good_deletes",
-				  _ipmi_mc_name(mc),
+				  i_ipmi_mc_name(mc),
 				  &sel->sel_good_deletes);
 	ipmi_domain_stat_register(domain, "sel_delete_lost_reservation",
-				  _ipmi_mc_name(mc),
+				  i_ipmi_mc_name(mc),
 				  &sel->sel_delete_lost_reservation);
 	ipmi_domain_stat_register(domain, "sel_fail_delete_lost_reservation",
-				  _ipmi_mc_name(mc),
+				  i_ipmi_mc_name(mc),
 				  &sel->sel_fail_delete_lost_reservation);
 	ipmi_domain_stat_register(domain, "sel_delete_errors",
-				  _ipmi_mc_name(mc),
+				  i_ipmi_mc_name(mc),
 				  &sel->sel_delete_errors);
 	*new_sel = sel;
     }
@@ -1110,8 +1110,8 @@ start_fetch_cb(ipmi_mc_t *mc, void *cb_data)
 	cmd_msg.netfn = IPMI_STORAGE_NETFN;
 	cmd_msg.cmd = IPMI_RESERVE_SEL_CMD;
 	cmd_msg.data_len = 0;
-	rv = ipmi_mc_send_command(mc, sel->lun, &cmd_msg,
-				  sel_handle_reservation, elem);
+	rv = ipmi_mc_send_command_sideeff(mc, sel->lun, &cmd_msg,
+					  sel_handle_reservation, elem);
     } else {
 	/* Bypass the reservation, it's not supported. */
 	sel->reservation = 0;
@@ -1740,8 +1740,8 @@ send_reserve_sel_for_delete(sel_del_handler_data_t *data, ipmi_mc_t *mc)
     cmd_msg.netfn = IPMI_STORAGE_NETFN;
     cmd_msg.cmd = IPMI_RESERVE_SEL_CMD;
     cmd_msg.data_len = 0;
-    rv = ipmi_mc_send_command(mc, data->lun,
-			      &cmd_msg, sel_reserved_for_delete, data);
+    rv = ipmi_mc_send_command_sideeff(mc, data->lun,
+				      &cmd_msg, sel_reserved_for_delete, data);
 
     return rv;
 }
@@ -1890,7 +1890,8 @@ sel_del_event_cb(ipmi_mc_t *mc, void *cb_data)
 	elem = opq_alloc_elem();
 	if (!data || !elem) {
 	    if (! info->do_clear) {
-		real_holder->deleted = 0;
+		if (real_holder)
+		    real_holder->deleted = 0;
 		sel->num_sels++;
 		sel->del_sels--;
 	    }
@@ -2196,25 +2197,22 @@ ipmi_get_all_sels(ipmi_sel_info_t *sel,
 	    rv = EINVAL;
 	    goto out_unlock;
 	}
-	for (i=0; ; ) {
+	for (i = 0; ; ) {
 	    sel_event_holder_t *holder = ilist_get(&iter);
 
-	    if (! holder->deleted) {
-		array[i] = ipmi_event_dup(holder->event);
-	    }
-	    i++;
+	    if (!holder->deleted)
+		array[i++] = ipmi_event_dup(holder->event);
 	    if (i < sel->num_sels) {
 		if (! ilist_next(&iter)) {
 		    rv = EINVAL;
-		    i--;
-		    while (i >= 0)
-			ipmi_event_free(array[i]);
+		    while (i > 0)
+			ipmi_event_free(array[--i]);
 		    goto out_unlock;
 		}
 	    } else
 		break;
 	}
-	*array_size = sel->num_sels;
+	*array_size = i;
     }
 
  out_unlock:
