@@ -264,12 +264,6 @@ got_discrete_states(ipmi_sensor_t *sensor,
 	goto out;
     }
 
-    if (err) {
-	printf("Error 0x%x getting discrete states for sensor %s\n",
-	       err, sdata->name);
-	goto out;
-    }
-
     printf("Got state reading for sensor %s\n", sdata->name);
     if (ipmi_is_event_messages_enabled(states))
 	printf("  event messages enabled\n");
@@ -537,7 +531,7 @@ sensor_change(enum ipmi_update_e op,
 	switch (sdata->state_sup)
 	{
 	    case IPMI_EVENT_SUPPORT_NONE:
-	    case IPMI_EVENT_SUPPORT_GLOBAL_ENABLE:
+	    case IPMI_EVENT_SUPPORT_GLOBAL_DISABLE:
 		/* No events to set up. */
 		printf("Sensor %s has no event support\n", sdata->name);
 		goto get_thresh;
